# -*- coding: utf-8 -*-

import curses


class Widget:
    def __init__(self, width = 10, height = 10):
        self.parent = None
        self.top = None
        # placing - set by parent widget
        self.x = 0
        self.y = 0
        # size
        self.width = width
        self.height = height
        self.allowlayout = True
        # redraw request
        self._redraw = True
        self.focus = False
        self.hidden = False
        # event handlers
        self.event = {
            'resize' : [],
            'draw' : [],
            'keypress' : [],
            'mousedown' : [],
            'mouseup' : [],
            'mousemove' : [],
            'mousewheel' : []}


    def settop(self, top):
        self.top = top


    def newevent(self, event):
        self.event[event] = []


    def connect(self, event, handler):
        if event in list(self.event.keys()):
            self.event[event] += [handler]


    def disconnect(self, event, handler=None):
        if event in list(self.event.keys()):
            if handler:
                i = self.event[event].index(handler)
                del self.event[event][i]
            else:
                self.event[event] = []


    def handle(self, event, *args, **kwargs):
        for handler in self.event[event]:
            handler(*args, **kwargs)


    def emit(self, event, *args, **kwargs):
        getattr(self, event)(*args, **kwargs)


    def resize(self):
        self.handle('resize')


    def redraw(self, parent=True):
        self._redraw = True
        if parent and self.parent:
            self.parent._redraw = True


    def draw(self, screen, x=0, y=0):
        #if self._redraw:
        if not self.hidden:
            self.handle('draw', screen, x, y)
            #self._redraw = False


    ### focus


    def canfocus(self):
        return bool(self.event['keypress'])


    def setfocus(self):
        self.focus = True


    def unfocus(self):
        self.focus = False


    def grabfocus(self):
        if not self.focus:
            self.parent.grabfocus(self) # grab focus for me


    ###


    def keypress(self, keyname, char):
        if self.focus:
            self.handle('keypress', keyname, char)


    def mousedown(self, ev):
        self.handle('mousedown', ev)


    def mouseup(self, ev):
        self.handle('mouseup', ev)


    def mousemove(self, ev):
        self.handle('mousemove', ev)


    def mousewheel(self, ev):
        self.handle('mousewheel', ev)


    def enclose(self, x, y):
        if self.hidden:
            return False
        if x < self.x or y < self.y \
        or x >= self.x + self.width or y >= self.y + self.height:
            return False
        return  True


    def screentest(self, y, x):
        sy, sx = self.screenyx()
        if y < sy or x < sx or y >= sy + self.height or x >= sx + self.width:
            return False
        return True


    def screenyx(self):
        if self.parent:
            y,x = self.parent.screenyx()
            return self.y + y, self.x + x
        return self.y, self.x
