# -*- coding: utf-8 -*-

from .container import Container


class Window(Container):
    def __init__(self, width=40, height=10):
        Container.__init__(self, width, height)

        self.connect('draw', self.on_draw)
        self.connect('mousedown', self.on_mousedown)
        self.connect('mouseup', self.on_mouseup)
        self.connect('mousemove', self.on_mousemove)

        self.title = ' '


    def on_draw(self, screen, x, y):
        screen.frame(x, y, self.width, self.height)
        screen.fill(x+1, y+1, self.width-2, self.height-2)
    #    screen.addstr(self.title)


    def on_mousedown(self, ev):
        self.dragstart = (ev.wx, ev.wy)
        if ev.wx >= self.width - 1 and ev.wy >= self.height - 1:
            self.resizing = True
        else:
            self.resizing = False
        self.origsize = (self.width, self.height)
        #self.title += '%dP%d ' % (x,y)
        self.redraw(True)


    def on_mouseup(self, ev):
        #self.title += '%dR%d ' % (x,y)

        if self.resizing:
            self.width = self.origsize[0] + ev.wx - self.dragstart[0]
            self.height = self.origsize[1] + ev.wy - self.dragstart[1]
        else:
            self.x = ev.px - self.dragstart[0]
            self.y = ev.py - self.dragstart[1]

        self.redraw(True)


    def on_mousemove(self, ev):
     #   self.title += '%dM%d ' % (x,y)
        if ev.px == self.x + self.dragstart[0] \
        and ev.py == self.y + self.dragstart[1]:
            return

        #if x > self.parent.width-self.width:
         #   return

        if self.resizing:
            self.width = self.origsize[0] + ev.wx - self.dragstart[0]
            self.height = self.origsize[1] + ev.wy - self.dragstart[1]
        else:
            self.x = ev.px - self.dragstart[0]
            self.y = ev.py - self.dragstart[1]

        self.redraw(True)

